# -*- coding: utf-8 -*-
"""
    ml_scikit_learn
    ~~~~~~~~~~~~~~~

    Demonstration of ML using scikit-learn package
    A binary classification problem: Predict whether student will have a grade better then C or not

    :author: Jiri Spilka, 2019
"""

from matplotlib import pyplot as plt
import numpy as np
from sklearn.linear_model import SGDClassifier
from sklearn.metrics import accuracy_score, confusion_matrix
from sklearn.model_selection import GridSearchCV
from sklearn.pipeline import Pipeline
from sklearn.preprocessing import StandardScaler

import utils

df = utils.load_data_binary()

# create train (years 2015, 2016, 2017) and test dataset (2018)
df_train = df.loc[df['year'] < 2018, :]
df_test = df.loc[df['year'] == 2018, :]

X_train = df_train.loc[:, ['points_st1', 'points_st2']].astype(np.float).values
X_test = df_test.loc[:, ['points_st1', 'points_st2']].astype(np.float).values
y_train = df_train['y'].values
y_test = df_test['y'].values

print(f"training set {len(X_train)}, #y==0: {sum(y_train == 0)} #y==1: {sum(y_train == 1)}")
print(f"test set     {len(X_test)}, #y==0: {sum(y_test == 0)} #y==1: {sum(y_test == 1)}")

# learn the model parameters on training data
params = {'clf__alpha': [0.0001, 0.001, 0.1, 1, 10]}

pipe = Pipeline(memory=None,
                steps=[
                    ('scale', StandardScaler()),
                    ('clf', SGDClassifier(loss='log'))
                ])

grid_search = GridSearchCV(pipe, params, cv=2, scoring='accuracy')

grid_search.fit(X_train, y_train)

print("\nBest score: %0.3f" % grid_search.best_score_)

print("Best parameters set:")
best_parameters = grid_search.best_estimator_.get_params()
for param_name in sorted(params.keys()):
    print("\t%s: %r" % (param_name, best_parameters[param_name]))

print("\nResults of grid search:")
cv_res = grid_search.cv_results_
for param, mean_s, std_s in zip(cv_res['params'],
                                cv_res['mean_test_score'],
                                cv_res['std_test_score']):
    print('mean: {0:2.5f}, std: {1:2.5f}, params: {2}'.format(mean_s, std_s, param))

# predict on a test data
y_test_pred = grid_search.predict(X_test)

acc = accuracy_score(y_true=y_test, y_pred=y_test_pred)
cm = confusion_matrix(y_true=y_test, y_pred=y_test_pred)

print(f"\nAccuracy: {acc}")
print(f"Confusion matrix: \n {cm}")


h = .02
x = X_test
y = y_test
x_min, x_max = X_test[:, 0].min() - 1, X_test[:, 0].max() + 1
y_min, y_max = X_test[:, 1].min() - 1, X_test[:, 1].max() + 1
xx, yy = np.meshgrid(np.arange(x_min, x_max, h), np.arange(y_min, y_max, h))

Z = grid_search.predict(np.c_[xx.ravel(), yy.ravel()])
Z = Z.reshape(xx.shape)
plt.contourf(xx, yy, Z, cmap=plt.cm.coolwarm, alpha=0.5)

plt.scatter(X_test[y_test == 0, 0], X_test[y_test == 0, 1],
            c='blue', cmap=plt.cm.coolwarm, s=25, edgecolors='k', label='y = 0 (grade E, F, D)')
plt.scatter(X_test[y_test == 1, 0], X_test[y_test == 1, 1],
            c='red', cmap=plt.cm.coolwarm, s=25, edgecolors='k', label='y = 1 (grade C, B, A)')
plt.xlabel('points st1')
plt.xlabel('points st2')
plt.legend()

plt.show()
